@@ -21,7 +21,7 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: cache.h,v 1.4 2003/02/16 21:11:14 mschimek Exp $ */
+/* $Id: cache.h,v 1.5 2006/02/10 06:25:37 mschimek Exp $ */
 
 #ifndef CACHE_H
 #define CACHE_H
@@ -39,22 +39,20 @@ typedef struct node node;
 typedef struct list list;
 
 struct node {
-	node *			succ;
-	node *			pred;
+	node *			_succ;
+	node *			_pred;
 };
 
 struct list {
-	node *			head;
-	node *			null;
-	node *			tail;
-	int			members;
+	node			_head;
+	unsigned int		_n_members;
 };
 
 struct cache {
 	/* TODO: thread safe */
 	list			hash[HASH_SIZE];
 
-	int			npages;
+	int			n_pages;
 
 	/* TODO */
 	unsigned long		mem_used;
