@@ -18,14 +18,15 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: caption.c,v 1.18 2005/01/24 00:13:39 mschimek Exp $ */
+/* $Id: caption.c,v 1.23 2006/05/31 03:53:36 mschimek Exp $ */
+
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
 
-#include <stdio.h>
-#include <stdlib.h>
 #include <unistd.h>
-#include <string.h>
-#include <assert.h>
 
+#include "misc.h"
 #include "trigger.h"
 #include "format.h"
 #include "lang.h"
@@ -35,7 +36,6 @@
 
 #define elements(array) (sizeof(array) / sizeof(array[0]))
 
-#define XDS_DEBUG(x) /* x */
 #define ITV_DEBUG(x) /* x */
 #define XDS_SEP_DEBUG(x) /* x */
 #define XDS_SEP_DUMP(x) /* x */
@@ -86,45 +86,6 @@ language[8] = {
 #define UNUSED
 #endif
 
-static const char *
-map_type[] UNUSED = {
-	"unknown", "mono", "simulated stereo", "stereo",
-	"stereo surround", "data service", "unknown", "none"
-};
-
-static const char *
-sap_type[] UNUSED = {
-	"unknown", "mono", "video descriptions", "non-program audio",
-	"special effects", "data service", "unknown", "none"
-};
-
-static const char *
-cgmsa[] UNUSED = {
-	"copying permitted",
-	"-",
-	"one generation copy allowed",
-	"no copying permitted"
-};
-
-static const char *
-scrambling[] UNUSED = {
-	"no pseudo-sync pulse",
-	"pseudo-sync pulse on; color striping off",
-	"pseudo-sync pulse on; 2-line color striping on",
-	"pseudo-sync pulse on; 4-line color striping on"
-};
-
-static const char *
-month_names[] UNUSED = {
-	"0?", "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug",
-	"Sep", "Oct", "Nov", "Dec", "13?", "14?", "15?"
-};
-
-static const char *
-day_names[] UNUSED = {
-	"0?", "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"
-};
-
 static uint32_t hcrc[128];
 
 static void init_hcrc(void) __attribute__ ((constructor));
@@ -206,8 +167,6 @@ xds_decoder(vbi_decoder *vbi, int _class, int type,
 	switch (_class) {
 	case XDS_CURRENT: /* 0 */
 	case XDS_FUTURE: /* 1 */
-		XDS_DEBUG(printf((class == XDS_CURRENT) ? "Current " : "Next "));
-
 		if (!(vbi->event_mask & (VBI_EVENT_ASPECT | VBI_EVENT_PROG_INFO)))
 			return;
 
@@ -227,14 +186,6 @@ xds_decoder(vbi_decoder *vbi, int _class, int type,
 			hour = buffer[1] & 31;
 			min = buffer[0] & 63;
 
-			XDS_DEBUG(printf("PIN: %d %s %02d:%02d UTC, "
-					 "D=%d L=%d Z=%d T(ape delayed)=%d\n",
-					 day, month_names[month], hour, min,
-					 !!(buffer[1] & 0x20),
-					 !!(buffer[2] & 0x20),
-					 !!(buffer[3] & 0x20),
-					 !!(buffer[3] & 0x10)));
-
 			if (month == 0 || month > 12
 			    || day == 0 || day > 31
 			    || hour > 23 || min > 59)
@@ -280,14 +231,6 @@ xds_decoder(vbi_decoder *vbi, int _class, int type,
 					esec = buffer[4] & 63;
 			}
 
-			XDS_DEBUG(
-				printf("length: %02d:%02d, ", lhour, lmin);
-				printf("elapsed: %02d:%02d", ehour, emin);
-				if (length >= 5)
-					printf(":%02d", esec);
-				printf("\n");
-			)
-
 			if (lmin > 59 || emin > 59 || esec > 59)
 				return;
 
@@ -304,13 +247,6 @@ xds_decoder(vbi_decoder *vbi, int _class, int type,
 			if (length < 2)
 				return;
 
-			XDS_DEBUG(
-				printf("program title: '");
-				for (i = 0; i < length; i++)
-					putchar(vbi_printable (buffer[i]));
-				printf("'\n");
-			)
-
 			neq = xds_strfu(pi->title, buffer, length);
 
 			if (!neq) { /* no title change */
@@ -333,15 +269,6 @@ xds_decoder(vbi_decoder *vbi, int _class, int type,
 		{
 			int neq;
 
-			XDS_DEBUG(
-				printf("program type: ");
-				for (i = 0; i < length; i++)
-					printf((i > 0) ? ", %s" : "%s",
-						vbi_prog_type_str_by_id(
-						 VBI_PROG_CLASSF_EIA_608, buffer[i]));
-				printf("\n");
-			)
-
 			neq = (pi->type_classf != VBI_PROG_CLASSF_EIA_608);
 			pi->type_classf = VBI_PROG_CLASSF_EIA_608;
 
@@ -376,28 +303,6 @@ xds_decoder(vbi_decoder *vbi, int _class, int type,
 			if (buffer[1] & 0x20)
 				dlsv |= VBI_RATING_V;
 
-			XDS_DEBUG(
-				printf("program movie rating: %s, tv rating: ",
-					vbi_rating_str_by_id(VBI_RATING_AUTH_MPAA, r));
-				if (buffer[0] & 0x10) {
-					if (buffer[0] & 0x20)
-						puts(vbi_rating_str_by_id(VBI_RATING_AUTH_TV_CA_FR, g));
-					else
-						puts(vbi_rating_str_by_id(VBI_RATING_AUTH_TV_CA_EN, g));
-				} else {
-					printf("%s; ", vbi_rating_str_by_id(VBI_RATING_AUTH_TV_US, g));
-					if (buffer[1] & 0x20)
-						printf("violence; ");
-					if (buffer[1] & 0x10)
-						printf("sexual situations; ");
-					if (buffer[1] & 8)
-						printf("indecent language; ");
-					if (buffer[0] & 0x20)
-						printf("sexually suggestive dialog");
-					putchar('\n');
-				}
-			)
-
 			if ((buffer[0] & 0x08) == 0) {
 				if (r == 0) return;
 				auth = VBI_RATING_AUTH_MPAA;
@@ -455,11 +360,6 @@ xds_decoder(vbi_decoder *vbi, int _class, int type,
 			if (length != 2)
 				return;
 
-			XDS_DEBUG(printf("main audio: %s, %s; "
-					 "second audio program: %s, %s\n",
-					 map_type[buffer[0] & 7], language[(buffer[0] >> 3) & 7],
-					 sap_type[buffer[1] & 7], language[(buffer[1] >> 3) & 7]));
-
 			for (i = 0; i < 2; i++) {
 				int l = (buffer[i] >> 3) & 7;
 				vbi_audio_mode m = mode[i][buffer[i] & 7];
@@ -493,12 +393,16 @@ xds_decoder(vbi_decoder *vbi, int _class, int type,
 			for (i = 0; i < length; i++) {
 				int ch = buffer[i] & 7;
 				int l = (buffer[i] >> 3) & 7;
-				const char *s;
+				/* should be const char *, but I got that
+				   wrong and cannot change the public
+				   pi->audio[].language type now. */
+				unsigned char *s;
 
 				ch = (ch & 1) * 4 + (ch >> 1);
 
 				services |= 1 << ch;
-				s = ((1 << l) & 0xC1) ? NULL : language[l];
+				s = ((1 << l) & 0xC1) ? NULL :
+					(unsigned char *) language[l];
 
 				if (pi->caption_language[ch] != (unsigned char *) s) {
 					neq = 1; pi->caption_language[ch] = (unsigned char *) s;
@@ -511,16 +415,6 @@ xds_decoder(vbi_decoder *vbi, int _class, int type,
 
 			xds_intfu(pi->caption_services, services);
 
-			XDS_DEBUG(
-				printf("program caption services:\n");
-				for (i = 0; i < length; i++)
-					printf("Line %3d, channel %d, %s: %s\n",
-						(buffer[i] & 4) ? 284 : 21,
-						(buffer[i] & 2) ? 2 : 1,
-						(buffer[i] & 1) ? "text      " : "captioning",
-						language[(buffer[i] >> 3) & 7]);
-			)
-
 			break;
 		}
 
@@ -528,13 +422,6 @@ xds_decoder(vbi_decoder *vbi, int _class, int type,
 			if (length != 1)
 				return;
 
-			XDS_DEBUG(
-				printf("CGMS: %s", cgmsa[(buffer[0] >> 3) & 3]);
-				if (buffer[0] & 0x18)
-					printf("; %s", scrambling[(buffer[0] >> 1) & 3]);
-				printf("; analog source: %d", buffer[0] & 1);
-			)
-
 			xds_intfu(pi->cgms_a, buffer[0] & 63);
 
 			break;
@@ -558,12 +445,6 @@ xds_decoder(vbi_decoder *vbi, int _class, int type,
 			else
 				r->ratio = 1.0;
 
-			XDS_DEBUG(printf("program aspect ratio info: "
-					 "active start %d, end %d%s\n",
-					 (buffer[0] & 63) + 22, 262 - (buffer[1] & 63),
-					 (length >= 3 && (buffer[2] & 1)) ?
-					 " (anamorphic)" : ""));
-
 			if (memcmp(r, &vbi->prog_info[0].aspect, sizeof(*r)) != 0) {
 				vbi->prog_info[0].aspect = *r;
 				vbi->aspect_source = 3;
@@ -581,21 +462,12 @@ xds_decoder(vbi_decoder *vbi, int _class, int type,
 		{
 			int line = type & 7;
 
-			XDS_DEBUG(
-				printf("program descr. line %d: >", line + 1);
-				for (i = 0; i < length; i++)
-					putchar(vbi_printable (buffer[i]));
-				printf("<\n");
-			)
-
 			neq = xds_strfu(pi->description[line], buffer, length);
 
 			break;
 		}
 
 		default:
-			XDS_DEBUG(printf("<unknown %d/%02x length %d>\n",
-					 class, type, length));
 			return; /* no event */
 		}
 
@@ -644,16 +516,12 @@ xds_decoder(vbi_decoder *vbi, int _class, int type,
 				vbi->network.type = VBI_EVENT_NETWORK;
 				caption_send_event(vbi, &vbi->network);
 
+				vbi->network.type = VBI_EVENT_NETWORK_ID;
+				caption_send_event(vbi, &vbi->network);
+
 				n->cycle = 3;
 			}
 
-			XDS_DEBUG(
-				printf("Network name: '");
-				for (i = 0; i < length; i++)
-					putchar(vbi_printable (buffer[i]));
-				printf("'\n");
-			)
-
 			break;
 
 		case 2:		/* network call letters */
@@ -664,13 +532,6 @@ xds_decoder(vbi_decoder *vbi, int _class, int type,
 				}
 			}
 
-			XDS_DEBUG(
-				printf("Network call letters: '");
-				for (i = 0; i < length; i++)
-					putchar(vbi_printable (buffer[i]));
-				printf("'\n");
-			)
-
 			break;
 
 		case 3:		/* channel tape delay */
@@ -680,14 +541,9 @@ xds_decoder(vbi_decoder *vbi, int _class, int type,
 			n->tape_delay =
 				(buffer[1] & 31) * 60 + (buffer[0] & 63);
 
-			XDS_DEBUG(printf("Channel tape delay: %02d:%02d",
-					 buffer[1] & 31, buffer[0] & 63));
-
 			break;
 
 		default:
-			XDS_DEBUG(printf("<unknown %d/%02x length %d>\n",
-					 class, type, length));
 			break;
 		}
 
@@ -698,73 +554,33 @@ xds_decoder(vbi_decoder *vbi, int _class, int type,
 		case 1:		/* time of day */
 			if (length != 6)
 				return;
-			XDS_DEBUG(
-				printf("Time of day (UTC): %s, %d %s %d ",
-					day_names[buffer[4] & 7],
-					buffer[2] & 31, month_names[buffer[3] & 15],
-					1990 + (buffer[5] & 63));
-				printf("%02d:%02d ", buffer[1] & 31, buffer[0] & 63);
-				printf("D(ST)=%d, L(eap day)=%d, "
-					"(Second )Z(ero)=%d, T(ape delayed)=%d\n",
-					!!(buffer[1] & 0x20),
-					!!(buffer[2] & 0x20),
-					!!(buffer[3] & 0x20),
-					!!(buffer[3] & 0x10));
-			)
 			break;
 
 		case 2:		/* impulse capture id */
 			if (length != 6)
 				return;
-			XDS_DEBUG(
-				printf("Impulse capture id: %d %s ",
-					buffer[2] & 31, month_names[buffer[3] & 15]);
-				printf("%02d:%02d ", buffer[1] & 31, buffer[0] & 63);
-				printf("length %02d:%02d ",
-					buffer[5] & 63, buffer[4] & 63);
-				printf("D=%d, L=%d, Z=%d, T(ape delayed)=%d\n",
-					!!(buffer[1] & 0x20),
-					!!(buffer[2] & 0x20),
-					!!(buffer[3] & 0x20),
-					!!(buffer[3] & 0x10));
-			)
 			break;
 
 		case 3:		/* supplemental data location */
-			XDS_DEBUG(
-				for (i = 0; i < length; i++)
-					printf("Supplemental data: field %d, line %d\n",
-					       !!(buffer[i] & 0x20), buffer[i] & 31);
-			)
 			break;
 
 		case 4:		/* local time zone */
 			if (length != 1)
 				return;
-			XDS_DEBUG(printf("Local time zone: UTC + %d h; D(ST)=%d\n",
-				buffer[0] & 31, !!(buffer[0] & 0x20)));
 			break;
 
 		case 0x40:	/* out-of-band channel number */
 			if (length != 2)
 				return;
-			XDS_DEBUG(
-				i = (buffer[0] & 63) | ((buffer[1] & 63) << 6);
-				printf("Out-of-band channel %d -- ?\n", i);
-			)
 			break;
 
 		default:
-			XDS_DEBUG(printf("<unknown %d/%02x length %d>\n",
-					 class, type, length));
 			break;
 		}
 
 		break;
 
 	default:
-		XDS_DEBUG(printf("<unknown %d/%02x length %d>\n",
-				 class, type, length));
 		break;
 	}
 }
@@ -835,7 +651,7 @@ xds_separator(vbi_decoder *vbi, uint8_t *buf)
 
 			XDS_SEP_DUMP(
 				for (i = 0; i < sp->count - 2; i++)
-					printf("%c", vbi_printable (sp->buffer[i]));
+					printf("%c", _vbi_to_ascii (sp->buffer[i]));
 				printf(" %d/0x%02x\n", class, type);
 			)
 		}
@@ -975,6 +791,8 @@ update(cc_channel *ch)
 static void
 word_break(struct caption *cc, cc_channel *ch, int upd)
 {
+	cc = cc;
+
 	/*
 	 *  Add a leading and trailing space.
 	 */
@@ -1439,8 +1257,8 @@ vbi_decode_caption(vbi_decoder *vbi, int line, uint8_t *buf)
 
 	case 284:	/* NTSC */
 		CC_DUMP(
-			putchar(vbi_printable (buf[0]));
-			putchar(vbi_printable (buf[1]));
+			putchar(_vbi_to_ascii (buf[0]));
+			putchar(_vbi_to_ascii (buf[1]));
 			fflush(stdout);
 		)
 
@@ -1475,8 +1293,8 @@ vbi_decode_caption(vbi_decoder *vbi, int line, uint8_t *buf)
 	}
 
 	CC_DUMP(
-		putchar(vbi_printable (buf[0]));
-		putchar(vbi_printable (buf[1]));
+		putchar(_vbi_to_ascii (buf[0]));
+		putchar(_vbi_to_ascii (buf[1]));
 		fflush(stdout);
 	)
 
@@ -1512,8 +1330,8 @@ vbi_decode_caption(vbi_decoder *vbi, int line, uint8_t *buf)
 
 	default:
 		CC_TEXT_DUMP(
-			putchar(vbi_printable (buf[0]));
-			putchar(vbi_printable (buf[1]));
+			putchar(_vbi_to_ascii (buf[0]));
+			putchar(_vbi_to_ascii (buf[1]));
 			fflush(stdout);
 		)
 
@@ -1767,6 +1585,8 @@ vbi_fetch_cc_page(vbi_decoder *vbi, vbi_page *pg, vbi_pgno pgno, vbi_bool reset)
 	cc_channel *ch = vbi->cc.channel + ((pgno - 1) & 7);
 	vbi_page *spg;
 
+	reset = reset;
+
 	if (pgno < 1 || pgno > 8)
 		return FALSE;
 
