@@ -17,14 +17,14 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: decoder.c,v 1.18 2005/01/24 00:13:39 mschimek Exp $ */
+/* $Id: decoder.c,v 1.22 2006/05/31 03:55:52 mschimek Exp $ */
 
 #include "site_def.h"
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <assert.h>
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
+
 #include <pthread.h>
 
 #include "misc.h"
@@ -469,6 +469,10 @@ vbi_bit_slicer_init(vbi_bit_slicer *slicer,
 	}
 }
 
+/**
+ * @example examples/wss.c
+ * WSS capture example.
+ */
 
 /**
  * @param rd Initialized vbi_raw_decoder structure.
@@ -739,8 +743,7 @@ vbi_raw_decoder_parameters	(vbi_raw_decoder *	rd,
 		service_set = vbi_sampling_par_from_services
 			((vbi_sampling_par *) rd,
 			 (unsigned int *) max_rate,
-			 videostd_set,
-			 service_set);
+			 videostd_set, service_set);
 	}
 
 	pthread_mutex_unlock(&rd->mutex);
