@@ -22,10 +22,10 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: exp-gfx.c,v 1.7 2002/10/22 04:42:40 mschimek Exp $ */
+/* $Id: exp-gfx.c,v 1.11 2006/02/10 06:25:37 mschimek Exp $ */
 
 #ifdef HAVE_CONFIG_H
-#  include "../config.h"
+#  include "config.h"
 #endif
 
 #include <stdio.h>
@@ -487,6 +487,7 @@ draw_blank(int canvas_type, uint8_t *canvas, unsigned int rowstride,
 }
 
 /**
+ * @param pg Source vbi_page, see vbi_fetch_cc_page().
  * @param fmt Target format. For now only VBI_PIXFMT_RGBA32_LE (vbi_rgba) permitted.
  * @param canvas Pointer to destination image (currently an array of vbi_rgba), this
  *   must be at least @a rowstride * @a height * 26 bytes large.
@@ -539,11 +540,17 @@ vbi_draw_cc_page_region(vbi_page *pg,
 			pen[0] = pg->color_map[ac->background];
 			pen[1] = pg->color_map[ac->foreground];
 
-			draw_char(sizeof(*canvast), (uint8_t *) canvast, rowstride,
-				  (uint8_t *) pen, ccfont2_bits, CCPL, CCW, CCH,
-				  unicode_ccfont2(ac->unicode, ac->italic), 0 /* bold */,
-				  ac->underline * (3 << 24) /* cell row 24, 25 */,
-				  VBI_NORMAL_SIZE);
+			draw_char (sizeof (*canvast),
+				   (uint8_t *) canvast,
+				   rowstride,
+				   (uint8_t *) pen,
+				   (uint8_t *) ccfont2_bits,
+				   CCPL, CCW, CCH,
+				   unicode_ccfont2 (ac->unicode, ac->italic),
+				   0 /* bold */,
+				   (ac->underline
+				    * (3 << 24)) /* cell row 24, 25 */,
+				   VBI_NORMAL_SIZE);
 
 			canvast += CCW;
 		}
@@ -639,10 +646,16 @@ vbi_draw_vt_page_region(vbi_page *pg,
 						draw_blank(sizeof(*canvast), (uint8_t *) canvast, rowstride,
 							   pen[0], TCW, TCH);
 				} else {
-					draw_char(sizeof(*canvast), (uint8_t *) canvast, rowstride,
-						(uint8_t *) pen, wstfont2_bits, TCPL, TCW, TCH,
-						unicode_wstfont2(unicode, ac->italic), ac->bold,
-						ac->underline << 9 /* cell row 9 */, ac->size);
+					draw_char (sizeof (*canvast),
+						   (uint8_t *) canvast,
+						   rowstride,
+						   (uint8_t *) pen,
+						   (uint8_t *) wstfont2_bits,
+						   TCPL, TCW, TCH,
+						   unicode_wstfont2 (unicode, ac->italic),
+						   ac->bold,
+						   ac->underline << 9 /* cell row 9 */,
+						   ac->size);
 				}
 			}
 
@@ -740,6 +753,8 @@ gfx_options[] = {
 static vbi_option_info *
 option_enum(vbi_export *e, int index)
 {
+	e = e;
+
 	if (index < 0 || index >= (int) elements(gfx_options))
 		return NULL;
 	else
@@ -927,7 +942,7 @@ draw_char_cc_indexed(png_bytep canvas, int rowstride, png_bytep pen,
 		     int unicode, vbi_char *ac)
 {
 	draw_char(sizeof(png_byte), (uint8_t *) canvas, rowstride,
-		  (uint8_t *) pen, ccfont2_bits, CCPL, CCW, CCH,
+		  (uint8_t *) pen, (uint8_t *) ccfont2_bits, CCPL, CCW, CCH,
 		  unicode_ccfont2(unicode, ac->italic), 0 /* bold */,
 		  ac->underline * (3 << 24) /* cell row 24, 25 */,
 		  VBI_NORMAL_SIZE);
@@ -938,7 +953,7 @@ draw_char_vt_indexed(png_bytep canvas, int rowstride, png_bytep pen,
 		     int unicode, vbi_char *ac)
 {
 	draw_char(sizeof(png_byte), (uint8_t *) canvas, rowstride,
-		  (uint8_t *) pen, wstfont2_bits, TCPL, TCW, TCH,
+		  (uint8_t *) pen, (uint8_t *) wstfont2_bits, TCPL, TCW, TCH,
 		  unicode_wstfont2(unicode, ac->italic), ac->bold,
 		  ac->underline << 9 /* cell row 9 */, ac->size);
 }
@@ -1119,7 +1134,8 @@ png_export(vbi_export *e, FILE *fp, vbi_page *pg)
 	}
 
 	/* avoid possible longjmp breakage due to libpng ugliness */
-	{ static int do_write() {
+	/* XXX not portable, to be removed */
+	{ int do_write() {
 	if (setjmp(png_ptr->jmpbuf))
 		return 1;
 
