@@ -21,15 +21,16 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: exp-txt.c,v 1.14 2004/12/13 07:13:32 mschimek Exp $ */
+/* $Id: exp-txt.c,v 1.18 2006/05/31 03:55:52 mschimek Exp $ */
+
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
 #include <limits.h>
 #include <iconv.h>
-#include <assert.h>
 
+#include "misc.h"
 #include "lang.h"
 #include "export.h"
 #include "exp-txt.h"
@@ -104,8 +105,9 @@ color_names[] __attribute__ ((unused)) = {
 };
 
 static const char *
-terminal[] __attribute__ ((unused)) = {
-	/* TRANSLATORS: Terminal control codes */
+terminal[] = {
+	/* TRANSLATORS:
+	   Terminal control codes. */
 	N_("None"), N_("ANSI X3.64 / VT 100"), N_("VT 200")
 };
 
@@ -140,6 +142,8 @@ text_options[] = {
 static vbi_option_info *
 option_enum(vbi_export *e, int index)
 {
+	e = e;
+
 	if (index < 0 || index >= (int) elements(text_options))
 		return NULL;
 
@@ -259,9 +263,9 @@ match_color8(vbi_rgba color)
 	int i, d, imin = 0, dmin = INT_MAX;
 
 	for (i = 0; i < 8; i++) {
-		d  = ABS(       (i & 1) * 0xFF - VBI_R(color));
-		d += ABS(((i >> 1) & 1) * 0xFF - VBI_G(color));
-		d += ABS( (i >> 2)      * 0xFF - VBI_B(color));
+		d  = ABS((int)(       (i & 1) * 0xFF - VBI_R(color)));
+		d += ABS((int)(((i >> 1) & 1) * 0xFF - VBI_G(color)));
+		d += ABS((int)( (i >> 2)      * 0xFF - VBI_B(color)));
 
 		if (d < dmin) {
 			dmin = d;
@@ -285,7 +289,8 @@ print_unicode(iconv_t cd, int endian, int unicode, char **p, int n)
 
 	r = iconv(cd, &ip, &li, &op, &lo);
 
-	if (r == -1 || (**p == 0x40 && unicode != 0x0040)) {
+	if ((size_t) -1 == r
+	    || (**p == 0x40 && unicode != 0x0040)) {
 		in[0 + endian] = 0x20;
 		in[1 - endian] = 0;
 		ip = in; op = *p;
@@ -293,7 +298,8 @@ print_unicode(iconv_t cd, int endian, int unicode, char **p, int n)
 
 		r = iconv(cd, &ip, &li, &op, &lo);
 
-		if (r == -1 || (r == 1 && **p == 0x40))
+		if ((size_t) -1 == r
+		    || (r == 1 && **p == 0x40))
 			goto error;
 	}
 
@@ -349,6 +355,8 @@ vbi_print_page_region(vbi_page *pg, char *buf, int size,
 	iconv_t cd;
 	char *p;
 
+	rtl = rtl;
+
 	if (1)
 		fprintf (stderr, "vbi_print_page_region '%s' "
 		         "table=%d col=%d row=%d width=%d height=%d\n",
