@@ -21,7 +21,7 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: export.c,v 1.21 2005/01/20 01:40:14 mschimek Exp $ */
+/* $Id: export.c,v 1.25 2006/05/25 08:10:03 mschimek Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
@@ -43,7 +43,7 @@ extern const char _zvbi_intl_domainname[];
 
 /**
  * @addtogroup Export Exporting formatted Teletext and Closed Caption pages
- * @ingroup Service
+ * @ingroup HiDec
  * 
  * Once libzvbi received, decoded and formatted a Teletext or Closed
  * Caption page you will want to render it on screen, print it as
@@ -84,7 +84,7 @@ extern const char _zvbi_intl_domainname[];
  * functions and export modules. libzvbi client applications
  * don't use this.
  *
- * Export modules @c #include @c "export.h" to get these
+ * Export modules include @c "export.h" to get these
  * definitions. See example module exp-templ.c.
  */
 
@@ -122,17 +122,17 @@ vbi_register_export_module(vbi_export_class *new_module)
 	*xcp = new_module;
 }
 
+extern vbi_export_class vbi_export_class_ppm;
+extern vbi_export_class vbi_export_class_png;
+extern vbi_export_class vbi_export_class_html;
+extern vbi_export_class vbi_export_class_tmpl;
+extern vbi_export_class vbi_export_class_text;
+extern vbi_export_class vbi_export_class_vtx;
+
 /* AUTOREG not reliable, sigh. */
 static void
 initialize(void)
 {
-	extern vbi_export_class vbi_export_class_ppm;
-	extern vbi_export_class vbi_export_class_png;
-	extern vbi_export_class vbi_export_class_html;
-	extern vbi_export_class vbi_export_class_tmpl;
-	extern vbi_export_class vbi_export_class_text;
-	extern vbi_export_class vbi_export_class_vtx;
-
 	static vbi_export_class *modules[] = {
 		&vbi_export_class_ppm,
 #ifdef HAVE_LIBPNG
@@ -548,7 +548,7 @@ vbi_export_new(const char *keyword, char **errstr)
 			break;
 
 	if (!xc) {
-		vbi_asprintf(errstr, _("Unknown export module '%s'."), key);
+		asprintf(errstr, _("Unknown export module '%s'."), key);
 		return NULL;
 	}
 
@@ -558,9 +558,9 @@ vbi_export_new(const char *keyword, char **errstr)
 		e = xc->_new();
 
 	if (!e) {
-		vbi_asprintf(errstr, _("Cannot initialize export module '%s', "
-				       "probably lack of memory."),
-			     xc->_public->label ? xc->_public->label : keyword);
+		asprintf(errstr, _("Cannot initialize export module '%s', "
+				   "probably lack of memory."),
+			 xc->_public->label ? xc->_public->label : keyword);
 		return NULL;
 	}
 
@@ -848,6 +848,7 @@ vbi_export_option_menu_get(vbi_export *export, const char *keyword,
 		case VBI_OPTION_REAL:
 			if (!oi->menu.dbl)
 				return FALSE;
+			/* XXX unsafe */
 			r = (oi->menu.dbl[i] == val.dbl);
 			break;
 
