@@ -68,14 +68,14 @@ static vbi_capture_dvb* dvb_init(const char *dev, char **errstr, int debug)
 
     dvb = malloc(sizeof(*dvb));
     if (NULL == dvb) {
-        vbi_asprintf(errstr, _("Virtual memory exhausted."));
+	asprintf(errstr, _("Virtual memory exhausted."));
         errno = ENOMEM;
 	return NULL;
     }
     CLEAR (*dvb);
 
     if (!(dvb->demux = vbi_dvb_pes_demux_new (NULL, NULL))) {
-	vbi_asprintf(errstr, _("Virtual memory exhausted."));
+	asprintf(errstr, _("Virtual memory exhausted."));
 	errno = ENOMEM;
         free (dvb);
 	return NULL;
@@ -84,8 +84,8 @@ static vbi_capture_dvb* dvb_init(const char *dev, char **errstr, int debug)
     dvb->debug = debug;
     dvb->fd = open(dev, O_RDWR | O_NONBLOCK);
     if (-1 == dvb->fd) {
-	vbi_asprintf(errstr, _("Cannot open '%s': %d, %s."),
-		     dev, errno, strerror(errno));
+	asprintf(errstr, _("Cannot open '%s': %d, %s."),
+		 dev, errno, strerror(errno));
 	free(dvb);
 	return NULL;
     }
@@ -327,6 +327,9 @@ static vbi_raw_decoder* dvb_parameters(vbi_capture *cap)
     static vbi_raw_decoder raw = {
 	.count = { 128, 128 },
     };
+
+    cap = cap;
+
     return &raw;
 }
 
@@ -410,8 +413,8 @@ vbi_capture_dvb_new2		(const char *		device_name,
 
     if (0 != pid) {
 	if (-1 == vbi_capture_dvb_filter (&dvb->cap, pid)) {
-	    vbi_asprintf (errstr, "DMX_SET_PES_FILTER: %s",
-			  strerror (errno));
+	    asprintf (errstr, "DMX_SET_PES_FILTER: %s",
+		      strerror (errno));
 	    dvb_delete (&dvb->cap);
 	    goto failure;
 	}
@@ -441,6 +444,10 @@ vbi_capture_dvb_new(char *dev, int scanning,
 	char *error = NULL;
 	vbi_capture *cap;
 
+	scanning = scanning;
+	services = services;
+	strict = strict;
+
 	if (!errstr)
 		errstr = &error;
 	*errstr = NULL;
@@ -484,6 +491,8 @@ vbi_capture_dvb_new(char *dev, int scanning,
 int64_t
 vbi_capture_dvb_last_pts	(const vbi_capture *	cap)
 {
+	cap = cap;
+
 	return 0;
 }
 
@@ -501,6 +510,9 @@ int
 vbi_capture_dvb_filter		(vbi_capture *		cap,
 				 int			pid)
 {
+	cap = cap;
+	pid = pid;
+
 	return -1;
 }
 
@@ -525,8 +537,12 @@ vbi_capture_dvb_new2		(const char *		device_name,
 				 char **		errstr,
 				 vbi_bool		trace)
 {
+	device_name = device_name;
+	pid = pid;
+	trace = trace;
+
 	if (errstr)
-		vbi_asprintf (errstr, _("DVB interface not compiled."));
+		asprintf (errstr, _("DVB interface not compiled."));
 
 	return NULL;
 }
@@ -566,8 +582,14 @@ vbi_capture_dvb_new(char *dev, int scanning,
 		    unsigned int *services, int strict,
 		    char **errstr, vbi_bool trace)
 {
+	dev = dev;
+	scanning = scanning;
+	services = services;
+	strict = strict;
+	trace = trace;
+
 	if (errstr)
-		vbi_asprintf (errstr, _("DVB interface not compiled."));
+		asprintf (errstr, _("DVB interface not compiled."));
 
 	return NULL;
 }
