@@ -17,7 +17,7 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: raw_decoder.c,v 1.5 2005/03/30 21:36:31 mschimek Exp $ */
+/* $Id: raw_decoder.c,v 1.15 2006/05/31 03:54:15 mschimek Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
@@ -39,11 +39,10 @@
 #  define RAW_DECODER_LOG 0
 #endif
 
-#define log(templ, args...)						\
+#define sp_log(level, templ, args...)					\
 do {									\
-	if (RAW_DECODER_LOG)						\
-		fprintf (stderr, "%s: " templ ".\n",			\
-			 __FUNCTION__ , ##args);			\
+	_vbi_log_printf (log_fn, log_user_data,				\
+			level, __FUNCTION__, templ , ##args);		\
 } while (0)
 
 /**
@@ -66,7 +65,8 @@ _vbi_service_table [] = {
 		{ 6, 318 },
 		{ 22, 335 },
 		10500, 6203125, 6203125, /* 397 x FH */
-		0x00AAAAE7, 0xFFFF, 18, 6, 37 * 8, VBI_MODULATION_NRZ_LSB
+		0x00AAAAE7, 0xFFFF, 18, 6, 37 * 8, VBI_MODULATION_NRZ_LSB,
+		0, /* probably */
 	}, {
 		VBI_SLICED_TELETEXT_B_L10_625,
 		"Teletext System B 625 Level 1.5",
@@ -74,7 +74,8 @@ _vbi_service_table [] = {
 		{ 7, 320 },
 		{ 22, 335 },
 		10300, 6937500, 6937500, /* 444 x FH */
-		0x00AAAAE4, 0xFFFF, 18, 6, 42 * 8, VBI_MODULATION_NRZ_LSB
+		0x00AAAAE4, 0xFFFF, 18, 6, 42 * 8, VBI_MODULATION_NRZ_LSB,
+		0,
 	}, {
 		VBI_SLICED_TELETEXT_B,
 		"Teletext System B, 625",
@@ -82,7 +83,8 @@ _vbi_service_table [] = {
 		{ 6, 318 },
 		{ 22, 335 },
 		10300, 6937500, 6937500, /* 444 x FH */
-		0x00AAAAE4, 0xFFFF, 18, 6, 42 * 8, VBI_MODULATION_NRZ_LSB
+		0x00AAAAE4, 0xFFFF, 18, 6, 42 * 8, VBI_MODULATION_NRZ_LSB,
+		0,
 	}, {
 		VBI_SLICED_TELETEXT_C_625, /* UNTESTED */
 		"Teletext System C 625",
@@ -90,7 +92,8 @@ _vbi_service_table [] = {
 		{ 6, 318 },
 		{ 22, 335 },
 		10480, 5734375, 5734375, /* 367 x FH */
-		0x00AAAAE7, 0xFFFF, 18, 6, 33 * 8, VBI_MODULATION_NRZ_LSB
+		0x00AAAAE7, 0xFFFF, 18, 6, 33 * 8, VBI_MODULATION_NRZ_LSB,
+		0,
 	}, {
 		VBI_SLICED_TELETEXT_D_625, /* UNTESTED */
 		"Teletext System D 625",
@@ -99,50 +102,60 @@ _vbi_service_table [] = {
 		{ 22, 335 },
 		10500, /* or 10970 depending on field order */
 		5642787, 5642787, /* 14/11 x FSC (color subcarrier) */
-		0x00AAAAE5, 0xFFFF, 18, 6, 34 * 8, VBI_MODULATION_NRZ_LSB
+		0x00AAAAE5, 0xFFFF, 18, 6, 34 * 8, VBI_MODULATION_NRZ_LSB,
+		0,
 	}, {
 		VBI_SLICED_VPS, "Video Program System",
 		VBI_VIDEOSTD_SET_PAL_BG,
 		{ 16, 0 },
 		{ 16, 0 },
 		12500, 5000000, 2500000, /* 160 x FH */
-		0xAAAA8A99, 0xFFFFFF, 32, 0, 13 * 8, VBI_MODULATION_BIPHASE_MSB
+		0xAAAA8A99, 0xFFFFFF, 32, 0, 13 * 8, VBI_MODULATION_BIPHASE_MSB,
+		_VBI_SP_FIELD_NUM,
 	}, {
 		VBI_SLICED_VPS_F2, "Pseudo-VPS on field 2",
 		VBI_VIDEOSTD_SET_PAL_BG,
 		{ 0, 329 },
 		{ 0, 329 },
 		12500, 5000000, 2500000, /* 160 x FH */
-		0xAAAA8A99, 0xFFFFFF, 32, 0, 13 * 8, VBI_MODULATION_BIPHASE_MSB
+		0xAAAA8A99, 0xFFFFFF, 32, 0, 13 * 8, VBI_MODULATION_BIPHASE_MSB,
+		_VBI_SP_FIELD_NUM,
 	}, {
 		VBI_SLICED_WSS_625, "Wide Screen Signalling 625",
 		VBI_VIDEOSTD_SET_625_50,
 		{ 23, 0 },
 		{ 23, 0 },
 		11000, 5000000, 833333, /* 160/3 x FH */
-		0xC71E3C1F, 0x924C99CE, 32, 0, 14 * 1,
-		VBI_MODULATION_BIPHASE_LSB
+		/* ...1000 111 / 0 0011 1100 0111 1000 0011 111x */
+		/* ...0010 010 / 0 1001 1001 0011 0011 1001 110x */	
+		0x8E3C783E, 0x2499339C, 32, 0, 14 * 1,
+		VBI_MODULATION_BIPHASE_LSB,
+		/* Hm. Too easily confused with caption?? */
+		_VBI_SP_FIELD_NUM | _VBI_SP_LINE_NUM,
 	}, {
 		VBI_SLICED_CAPTION_625_F1, "Closed Caption 625, field 1",
 		VBI_VIDEOSTD_SET_625_50,
 		{ 22, 0 },
 		{ 22, 0 },
 		10500, 1000000, 500000, /* 32 x FH */
-		0x00005551, 0x7FF, 14, 2, 2 * 8, VBI_MODULATION_NRZ_LSB
+		0x00005551, 0x7FF, 14, 2, 2 * 8, VBI_MODULATION_NRZ_LSB,
+		_VBI_SP_FIELD_NUM,
 	}, {
 		VBI_SLICED_CAPTION_625_F2, "Closed Caption 625, field 2",
 		VBI_VIDEOSTD_SET_625_50,
 		{ 0, 335 },
 		{ 0, 335 },
 		10500, 1000000, 500000, /* 32 x FH */
-		0x00005551, 0x7FF, 14, 2, 2 * 8, VBI_MODULATION_NRZ_LSB
+		0x00005551, 0x7FF, 14, 2, 2 * 8, VBI_MODULATION_NRZ_LSB,
+		_VBI_SP_FIELD_NUM,
 	}, {
 		VBI_SLICED_VBI_625, "VBI 625", /* Blank VBI */
 		VBI_VIDEOSTD_SET_625_50,
 		{ 6, 318 },
 		{ 22, 335 },
 		10000, 1510000, 1510000,
-		0, 0, 0, 0, 10 * 8, 0 /* 10.0-2 ... 62.9+1 us */
+		0, 0, 0, 0, 10 * 8, 0, /* 10.0-2 ... 62.9+1 us */
+		0,
 	}, {
 		VBI_SLICED_TELETEXT_B_525, /* UNTESTED */
 		"Teletext System B 525",
@@ -150,7 +163,8 @@ _vbi_service_table [] = {
 		{ 10, 272 },
 		{ 21, 284 },
 		10500, 5727272, 5727272, /* 364 x FH */
-		0x00AAAAE4, 0xFFFF, 18, 6, 34 * 8, VBI_MODULATION_NRZ_LSB
+		0x00AAAAE4, 0xFFFF, 18, 6, 34 * 8, VBI_MODULATION_NRZ_LSB,
+		0,
 	}, {
 		VBI_SLICED_TELETEXT_C_525, /* UNTESTED */
 		"Teletext System C 525",
@@ -158,7 +172,8 @@ _vbi_service_table [] = {
 		{ 10, 272 },
 		{ 21, 284 },
 		10480, 5727272, 5727272, /* 364 x FH */
-		0x00AAAAE7, 0xFFFF, 18, 6, 33 * 8, VBI_MODULATION_NRZ_LSB
+		0x00AAAAE7, 0xFFFF, 18, 6, 33 * 8, VBI_MODULATION_NRZ_LSB,
+		0,
 	}, {
 		VBI_SLICED_TELETEXT_D_525, /* UNTESTED */
 		"Teletext System D 525",
@@ -166,7 +181,8 @@ _vbi_service_table [] = {
 		{ 10, 272 },
 		{ 21, 284 },
 		9780, 5727272, 5727272, /* 364 x FH */
-		0x00AAAAE5, 0xFFFF, 18, 6, 34 * 8, VBI_MODULATION_NRZ_LSB
+		0x00AAAAE5, 0xFFFF, 18, 6, 34 * 8, VBI_MODULATION_NRZ_LSB,
+		0,
 	}, {
 #if 0 /* FIXME probably wrong */
 		VBI_SLICED_WSS_CPR1204,	/* NOT CONFIRMED (EIA-J CPR-1204) */
@@ -175,8 +191,9 @@ _vbi_service_table [] = {
 		{ 20, 283 },
 		{ 20, 283 },
 		11200, 1789773, 447443, /* 1/8 x FSC */
-		0x000000F0, 0xFF, 8, 0, 20 * 1, VBI_MODULATION_NRZ_MSB
+		0x000000F0, 0xFF, 8, 0, 20 * 1, VBI_MODULATION_NRZ_MSB,
 		/* No useful FRC, but a six bit CRC */
+		0,
 	}, {
 #endif
 		VBI_SLICED_CAPTION_525_F1,
@@ -185,7 +202,10 @@ _vbi_service_table [] = {
 		{ 21, 0 },
 		{ 21, 0 },
 		10500, 1006976, 503488, /* 32 x FH */
-		0x00005551, 0x7FF, 14, 2, 2 * 8, VBI_MODULATION_NRZ_LSB
+		0x00005551, 0x7FF, 14, 2, 2 * 8, VBI_MODULATION_NRZ_LSB,
+		/* I've seen CC signals on other lines and there's no
+		   way to distinguish from the transmitted data. */
+		_VBI_SP_FIELD_NUM | _VBI_SP_LINE_NUM,
 	}, {
 		VBI_SLICED_CAPTION_525_F2,
 		"Closed Caption 525, field 2",
@@ -193,7 +213,8 @@ _vbi_service_table [] = {
 		{ 0, 284 },
 		{ 0, 284 },
 		10500, 1006976, 503488, /* 32 x FH */
-		0x00005551, 0x7FF, 14, 2, 2 * 8, VBI_MODULATION_NRZ_LSB
+		0x00005551, 0x7FF, 14, 2, 2 * 8, VBI_MODULATION_NRZ_LSB,
+		_VBI_SP_FIELD_NUM | _VBI_SP_LINE_NUM,
 	}, {
 		VBI_SLICED_2xCAPTION_525, /* NOT CONFIRMED */
 		"2xCaption 525",
@@ -202,21 +223,24 @@ _vbi_service_table [] = {
 		{ 21, 0 },
 		10500, 1006976, 1006976, /* 64 x FH */
 		0x000554ED, 0xFFFF, 12, 8, 4 * 8,
-		VBI_MODULATION_NRZ_LSB /* Tb. */
+		VBI_MODULATION_NRZ_LSB, /* Tb. */
+		_VBI_SP_FIELD_NUM,
 	}, {
 		VBI_SLICED_VBI_525, "VBI 525", /* Blank VBI */
 		VBI_VIDEOSTD_SET_525_60,
 		{ 10, 272 },
 		{ 21, 284 },
 		9500, 1510000, 1510000,
-		0, 0, 0, 0, 10 * 8, 0 /* 9.5-1 ... 62.4+1 us */
+		0, 0, 0, 0, 10 * 8, 0, /* 9.5-1 ... 62.4+1 us */
+		0,
 	}, {
 		0, NULL,
 		VBI_VIDEOSTD_SET_EMPTY,
 		{ 0, 0 },
 		{ 0, 0 },
 		0, 0, 0,
-		0, 0, 0, 0, 0, 0
+		0, 0, 0, 0, 0, 0,
+		0,
 	}
 };
 
@@ -264,12 +288,12 @@ vbi_sliced_name			(vbi_service_set	service)
 }
 
 /**
- * $ingroup Sliced
- * $param service A data service identifier, for example from a
+ * @ingroup Sliced
+ * @param service A data service identifier, for example from a
  *   vbi_sliced structure.
  *
- * $return
- * Number of payload bits, $c 0 if the service is unknown.
+ * @return
+ * Number of payload bits, @c 0 if the service is unknown.
  */
 unsigned int
 vbi_sliced_payload_bits		(unsigned int		service)
@@ -442,17 +466,16 @@ decode_pattern			(vbi3_raw_decoder *	rd,
 			/* Positive match, output decoded line. */
 
 			sliced->id = job->id;
+			sliced->line = 0;
 
 			if (i >= (unsigned int) sp->count[0]) {
-				if (0 == sp->start[1])
-					sliced->line = 0;
-				else
+				if (sp->synchronous
+				    && 0 != sp->start[1])
 					sliced->line = sp->start[1]
 						+ i - sp->count[0];
 			} else {
-				if (0 == sp->start[0])
-					sliced->line = 0;
-				else
+				if (sp->synchronous
+				    && 0 != sp->start[0])
 					sliced->line = sp->start[0] + i;
 			}
 
@@ -577,394 +600,6 @@ vbi3_raw_decoder_decode		(vbi3_raw_decoder *	rd,
 	return sliced - sliced_begin;
 }
 
-vbi_bool
-_vbi_sampling_par_verify	(const vbi_sampling_par *sp)
-{
-	switch (sp->sampling_format) {
-	case VBI_PIXFMT_YUV420:
-		/* This conflicts with the ivtv driver, which returns an
-		   odd number of bytes per line.  The driver format is
-		   _GREY but libzvbi 0.2 has no VBI_PIXFMT_Y8. */
-		/* if (sp->bytes_per_line & 1)
-		   goto samples; */
-		break;
-
-	default:
-		if (0 != (sp->bytes_per_line
-			  % VBI_PIXFMT_BPP (sp->sampling_format)))
-			goto samples;
-		break;
-	}
-
-	if (0 == sp->count[0]
-	    && 0 == sp->count[1])
-		goto range;
-
-	if (525 == sp->scanning) {
-		if (0 != sp->start[0])
-			if ((sp->start[0] + sp->count[0]) > 265)
-				goto range;
-
-		if (0 != sp->start[1])
-			if (sp->start[1] < 263
-			    || (sp->start[1] + sp->count[0]) > 526)
-				goto range;
-	} else if (625 == sp->scanning) {
-		if (0 != sp->start[0])
-			if ((sp->start[0] + sp->count[0]) > 336)
-				goto range;
-
-		if (0 != sp->start[1])
-			if (sp->start[1] < 310
-			    || (sp->start[1] + sp->count[0]) > 626)
-				goto range;
-	} else {
-		log ("Ambiguous scanning %d",
-		     sp->scanning);
-		return FALSE;
-	}
-
-	if (sp->interlaced
-	    && sp->count[0] != sp->count[1]) {
-		log ("Line count %u, %u must be equal and non-zero "
-		     "when interlaced",
-		     sp->count[0], sp->count[1]);
-		return FALSE;
-	}
-
-	return TRUE;
-
- samples:
-	log ("Misaligned bytes_per_line %u",
-	     sp->bytes_per_line);
-
-	return FALSE;
-
- range:
-	log ("Invalid scan range %u-%u (%u lines), %u-%u (%u lines)",
-	     sp->start[0], sp->start[0] + sp->count[0] - 1, sp->count[0],
-	     sp->start[1], sp->start[1] + sp->count[1] - 1, sp->count[1]);
-
-	return FALSE;
-}
-
-/* Attn: strict must be int for compatibility with libzvbi 0.2 (-1 == 0) */
-static vbi_bool
-_vbi_sampling_par_check_service	(const vbi_sampling_par *sp,
-				 const _vbi_service_par *par,
-				 int			strict)
-{
-	double signal;
-	unsigned int field;
-	unsigned int samples_per_line;
-	vbi_videostd_set videostd_set;
-
-	switch (sp->scanning) {
-	case 525:
-		videostd_set = VBI_VIDEOSTD_SET_525_60;
-		break;
-
-	case 625:
-		videostd_set = VBI_VIDEOSTD_SET_625_50;
-		break;
-
-	default:
-		videostd_set = 0;
-		break;
-	}
-
-	if (0 == (par->videostd_set & videostd_set)) {
-		log ("Service 0x%08x (%s) requires "
-		     "videostd_set 0x%08llx, have 0x%08llx",
-		     par->id, par->label,
-		     par->videostd_set, videostd_set);
-		return FALSE;
-	}
-
-	/* I've seen CC signals on other lines and there's no
-	   way to distinguish from the transmitted data. */
-	if ((VBI_SLICED_CAPTION_525 & par->id)
-	    && (0 == sp->start[0] /* unknown */
-		|| 0 == sp->start[1])) {
-		log ("Service 0x%08x (%s) requires exact line numbers",
-		     par->id, par->label);
-		return FALSE;
-	}
-
-	{
-		unsigned int rate;
-
-		rate = MAX (par->cri_rate, par->bit_rate);
-
-		switch (par->id) {
-		case VBI_SLICED_WSS_625:
-			/* Effective bit rate is just 1/3 max_rate,
-			   so 1 * max_rate should suffice. */
-			break;
-
-		default:
-			rate = (rate * 3) >> 1;
-			break;
-		}
-
-		if (rate > (unsigned int) sp->sampling_rate) {
-			log ("Sampling rate %f MHz too low "
-			     "for service 0x%08x (%s)",
-			     sp->sampling_rate / 1e6,
-			     par->id, par->label);
-			return FALSE;
-		}
-	}
-
-	signal = par->cri_bits / (double) par->cri_rate
-		+ (par->frc_bits + par->payload) / (double) par->bit_rate;
-
-	samples_per_line = sp->bytes_per_line
-		/ VBI_PIXFMT_BPP (sp->sampling_format);
-
-	if (sp->offset > 0 && strict > 0) {
-		double sampling_rate;
-		double offset;
-		double end;
-
-		sampling_rate = (double) sp->sampling_rate;
-
-		offset = sp->offset / sampling_rate;
-		end = (sp->offset + samples_per_line) / sampling_rate;
-
-		if (offset > (par->offset / 1e3 - 0.5e-6)) {
-			log ("Sampling offset %f us too late "
-			     "for service 0x%08x (%s) at %f us",
-			     offset * 1e6,
-			     par->id, par->label,
-			     par->offset / 1e3);
-			return FALSE;
-		}
-
-		if (end < (par->offset / 1e9 + signal + 0.5e-6)) {
-			log ("Sampling ends too early at %f us "
-			     "for service 0x%08x (%s) up to %f us",
-			     end * 1e6,
-			     par->id, par->label,
-			     par->offset / 1e3
-			     + signal * 1e6 + 0.5);
-			return FALSE;
-		}
-	} else {
-		double samples;
-
-		samples = samples_per_line / (double) sp->sampling_rate;
-
-		if (samples < (signal + 1.0e-6)) {
-			log ("Service 0x%08x (%s) signal %f us "
-			     "exceeds %f us sampling period",
-			     par->id, par->label,
-			     signal * 1e6, samples * 1e6);
-			return FALSE;
-		}
-	}
-
-	if (!sp->synchronous) {
-		/* Not always, but it's too difficult now to bother. */
-		log ("Service 0x%08x (%s) requires "
-		     "synchronous field order",
-		     par->id, par->label);
-		return FALSE;
-	}
-
-	for (field = 0; field < 2; ++field) {
-		unsigned int start;
-		unsigned int end;
-
-		start = sp->start[field];
-		end = start + sp->count[field] - 1;
-
-		if (0 == par->first[field]
-		    || 0 == par->last[field]) {
-			/* No data on this field. */
-			continue;
-		}
-
-		if (0 == sp->count[field]) {
-			log ("Service 0x%08x (%s) requires "
-			     "data from field %u",
-			     par->id, par->label, field);
-			return FALSE;
-		}
-
-		if (sp->start[field] > 0 && strict > 0) {
-			/* May succeed if not all scanning lines
-			   available for the service are actually used. */
-			if (strict > 1
-			    || (par->first[field] == par->last[field]))
-				if (start > par->first[field]
-				    || end < par->last[field]) {
-					log ("Service 0x%08x (%s) requires "
-					     "lines %u-%u, have %u-%u",
-					     par->id, par->label,
-					     par->first[field],
-					     par->last[field],
-					     start, end);
-					return FALSE;
-				}
-		}
-	}
-
-	return TRUE;
-}
-
-/* Attn: strict must be int for compatibility with libzvbi 0.2 (-1 == 0) */
-vbi_service_set
-vbi_sampling_par_check_services	(const vbi_sampling_par *sp,
-				 vbi_service_set	services,
-				 int			strict)
-{
-	const _vbi_service_par *par;
-	vbi_service_set rservices;
-
-	assert (NULL != sp);
-
-	rservices = 0;
-
-	for (par = _vbi_service_table; par->id; ++par) {
-		if (par->id & services)
-			if (_vbi_sampling_par_check_service (sp, par, strict))
-				rservices |= par->id;
-	}
-
-	return rservices;
-}
-
-vbi_service_set
-vbi_sampling_par_from_services	(vbi_sampling_par *	sp,
-				 unsigned int *		max_rate,
-				 vbi_videostd_set	videostd_set,
-				 vbi_service_set	services)
-{
-	const _vbi_service_par *par;
-	vbi_service_set rservices;
-	unsigned int rate;
-
-	assert (NULL != sp);
-
-	if (0 != videostd_set) {
-		if (0 == (VBI_VIDEOSTD_SET_ALL & videostd_set)
-		    || ((VBI_VIDEOSTD_SET_525_60 & videostd_set)
-			&& (VBI_VIDEOSTD_SET_625_50 & videostd_set))) {
-			log ("Ambiguous videostd_set 0x%08llx",
-			     videostd_set);
-			/* CLEAR (*sp); */
-			return 0;
-		}
-	}
-
-	sp->sampling_format	= VBI_PIXFMT_YUV420; /* VBI_PIXFMT_Y8 */
-	sp->sampling_rate	= 27000000;		/* ITU-R BT.601 */
-	sp->bytes_per_line	= 0;
-	sp->offset		= (int)(64e-6 * sp->sampling_rate);
-	sp->start[0]		= 30000;
-	sp->count[0]		= 0;
-	sp->start[1]		= 30000;
-	sp->count[1]		= 0;
-	sp->interlaced		= FALSE;
-	sp->synchronous		= TRUE;
-
-	rservices = 0;
-	rate = 0;
-
-	for (par = _vbi_service_table; par->id; ++par) {
-		double margin;
-		double signal;
-		int offset;
-		unsigned int samples;
-		unsigned int i;
-
-		if (0 == (par->id & services))
-			continue;
-
-		if (0 == videostd_set) {
-			vbi_videostd_set set;
-
-			set = par->videostd_set | videostd_set;
-
-			if (0 == (VBI_VIDEOSTD_SET_525_60 & set)
-			    || 0 == (VBI_VIDEOSTD_SET_625_50 & set))
-				videostd_set = set;
-		}
-
-		if (VBI_VIDEOSTD_SET_525_60 & videostd_set)
-			margin = 1.0e-6;
-		else
-			margin = 2.0e-6;
-
-		if (0 == (par->videostd_set & videostd_set)) {
-			log ("Service 0x%08x (%s) requires "
-			     "videostd_set 0x%08llx, have 0x%08llx",
-			     par->id, par->label,
-			     par->videostd_set, videostd_set);
-			continue;
-		}
-
-		rate = MAX (rate, par->cri_rate);
-		rate = MAX (rate, par->bit_rate);
-
-		signal = par->cri_bits / (double) par->cri_rate
-			+ ((par->frc_bits + par->payload) / (double) par->bit_rate);
-
-		offset = (int)((par->offset / 1e9) * sp->sampling_rate);
-		samples = (int)((signal + 1.0e-6) * sp->sampling_rate);
-
-		sp->offset = MIN (sp->offset, offset);
-
-		/* Note bytes_per_sample is 1. */
-		sp->bytes_per_line =
-			MAX ((unsigned int) sp->bytes_per_line + sp->offset,
-			     samples + offset) - sp->offset;
-
-		for (i = 0; i < 2; ++i)
-			if (par->first[i] > 0
-			    && par->last[i] > 0) {
-				unsigned int start;
-				unsigned int end;
-
-				start = (unsigned int) sp->start[i];
-				sp->start[i] = MIN (start, par->first[i]);
-
-				end = sp->start[i] + sp->count[i];
-				sp->count[i] = MAX (end, par->last[i] + 1)
-					- sp->start[i];
-			}
-
-		rservices |= par->id;
-	}
-
-	if (0 == rservices) {
-		/* CLEAR (*sp); */
-		return 0;
-	}
-
-	if (videostd_set & VBI_VIDEOSTD_SET_525_60)
-		sp->scanning = 525;
-	else
-		sp->scanning = 625;
-
-	if (0 == sp->count[1]) {
-		sp->start[1] = 0;
-
-		if (0 == sp->count[0]) {
-			sp->start[0] = 0;
-			sp->offset = 0;
-		}
-	} else if (0 == sp->count[0]) {
-		sp->start[0] = 0;
-	}
-
-	if (max_rate)
-		*max_rate = rate;
-
-	return rservices;
-}
-
 /**
  * $param rd Pointer to vbi3_raw_decoder object allocated with
  *   vbi3_raw_decoder_new().
@@ -1159,6 +794,55 @@ add_job_to_pattern		(vbi3_raw_decoder *	rd,
 	return TRUE;
 }
 
+static void
+lines_containing_data		(unsigned int		start[2],
+				 unsigned int		count[2],
+				 const vbi_sampling_par *sp,
+				 const _vbi_service_par *par)
+{
+	unsigned int field;
+
+	start[0] = 0;
+	start[1] = sp->count[0];
+
+	count[0] = sp->count[0];
+	count[1] = sp->count[1];
+
+	if (!sp->synchronous) {
+		/* XXX Scanning all lines isn't always necessary. */
+		return;
+	}
+
+	for (field = 0; field < 2; ++field) {
+		unsigned int first;
+		unsigned int last;
+
+		if (0 == par->first[field]
+		    || 0 == par->last[field]) {
+			/* No data on this field. */
+			count[field] = 0;
+			continue;
+		}
+
+		first = sp->start[field];
+		last = first + sp->count[field] - 1;
+
+		if (first > 0 && sp->count[field] > 0) {
+			assert (par->first[field] <= par->last[field]);
+
+			if ((unsigned int) par->first[field] > last
+			    || (unsigned int) par->last[field] < first)
+				continue;
+
+			first = MAX (first, (unsigned int) par->first[field]);
+			last = MIN ((unsigned int) par->last[field], last);
+
+			start[field] += first - sp->start[field];
+			count[field] = last + 1 - first;
+		}
+	}
+}
+
 /**
  * $param rd Pointer to vbi_raw_decoder object allocated with
  *   vbi3_raw_decoder_new().
@@ -1188,16 +872,19 @@ vbi3_raw_decoder_add_services	(vbi3_raw_decoder *	rd,
 	const _vbi_service_par *par;
 	double min_offset;
 
+	assert (NULL != rd);
+
 	services &= ~(VBI_SLICED_VBI_525 | VBI_SLICED_VBI_625);
 
 	if (rd->services & services) {
-		log ("Already decoding services 0x%08x",
-		     rd->services & services);
+		info (&rd->log,
+		      "Already decoding services 0x%08x.",
+		      rd->services & services);
 		services &= ~rd->services;
 	}
 
 	if (0 == services) {
-		log ("No services to add");
+		info (&rd->log, "No services to add.");
 		return rd->services;
 	}
 
@@ -1211,28 +898,28 @@ vbi3_raw_decoder_add_services	(vbi3_raw_decoder *	rd,
 		rd->pattern = (int8_t *)
 			malloc (scan_ways * sizeof (rd->pattern[0]));
 		if (!rd->pattern) {
-			log ("Out of memory");
+			error (&rd->log, "Out of memory.");
 			return rd->services;
 		}
 
 		memset (rd->pattern, 0, scan_ways * sizeof (rd->pattern[0]));
 	}
 
-	if (525 == rd->sampling.scanning)
+	if (525 == rd->sampling.scanning) {
 		min_offset = 7.9e-6;
-	else
+	} else {
 		min_offset = 8.0e-6;
+	}
 
 	for (par = _vbi_service_table; par->id; ++par) {
 		vbi_sampling_par *sp;
 		_vbi3_raw_decoder_job *job;
-		unsigned int j;
-		unsigned int field;
 		unsigned int start[2];
 		unsigned int count[2];
 		unsigned int sample_offset;
 		unsigned int samples_per_line;
 		unsigned int cri_end;
+		unsigned int j;
 
 		if (0 == (par->id & services))
 			continue;
@@ -1256,8 +943,11 @@ vbi3_raw_decoder_add_services	(vbi3_raw_decoder *	rd,
 		}
 
 		if (j >= _VBI3_RAW_DECODER_MAX_JOBS) {
-			log ("Set 0x%08x exceeds %u service limit",
-			     services, _VBI3_RAW_DECODER_MAX_WAYS);
+			error (&rd->log,
+			       "Set 0x%08x exceeds number of "
+			       "simultaneously decodable "
+			       "services (%u).",
+			       services, _VBI3_RAW_DECODER_MAX_WAYS);
 			break;
 		} else if (j >= rd->n_jobs) {
 			job->id = 0;
@@ -1266,7 +956,10 @@ vbi3_raw_decoder_add_services	(vbi3_raw_decoder *	rd,
 
 		sp = &rd->sampling;
 
-		if (!_vbi_sampling_par_check_service (sp, par, strict))
+		if (!_vbi_sampling_par_check_services_log (sp,
+							   par->id,
+							   strict,
+							   &rd->log))
 			continue;
 
 
@@ -1291,63 +984,40 @@ vbi3_raw_decoder_add_services	(vbi3_raw_decoder *	rd,
 			cri_end = ~0;
 		}
 
+		/* XXX use sp->samples_per_line if available */
 		samples_per_line = sp->bytes_per_line
 			/ VBI_PIXFMT_BPP (sp->sampling_format);
 
-		if (!_vbi3_bit_slicer_init (&job->slicer,
-					   sp->sampling_format,
-					   sp->sampling_rate,
-					   sample_offset,
-					   samples_per_line,
-					   par->cri_frc >> par->frc_bits,
-					   par->cri_frc_mask >> par->frc_bits,
-					   par->cri_bits,
-					   par->cri_rate,
-					   cri_end,
-					   (par->cri_frc
-					    & ((1U << par->frc_bits) - 1)),
-					   par->frc_bits,
-					   par->payload,
-					   par->bit_rate,
-					   par->modulation)) {
+		if (!_vbi3_bit_slicer_init (&job->slicer)) {
 			assert (!"bit_slicer_init");
 		}
 
-
-		for (field = 0; field < 2; ++field) {
-			unsigned int first;
-			unsigned int last;
-
-			if (0 == par->first[field]
-			    || 0 == par->last[field]) {
-				/* No data on this field. */
-				start[field] = 0;
-				count[field] = 0;
-				continue;
-			}
-
-			first = sp->start[field];
-			last = first + sp->count[field] - 1;
-
-			if (first > 0 && strict > 0) {
-				first = MAX (first,
-					     (unsigned int) par->first[field]);
-				start[field] = first - sp->start[field];
-				last = MIN (last,
-					    (unsigned int) par->last[field]);
-				count[field] = last - first + 1;
-			} else {
-				start[field] = 0;
-				count[field] = sp->count[field];
-			}
+		if (!vbi3_bit_slicer_set_params
+		    (&job->slicer,
+		     sp->sampling_format,
+		     sp->sampling_rate,
+		     sample_offset,
+		     samples_per_line,
+		     par->cri_frc >> par->frc_bits,
+		     par->cri_frc_mask >> par->frc_bits,
+		     par->cri_bits,
+		     par->cri_rate,
+		     cri_end,
+		     (par->cri_frc & ((1U << par->frc_bits) - 1)),
+		     par->frc_bits,
+		     par->payload,
+		     par->bit_rate,
+		     par->modulation)) {
+			assert (!"bit_slicer_set_params");
 		}
 
-		start[1] += sp->count[0]; /* rel. raw image, not scan lines */
+		lines_containing_data (start, count, sp, par);
 
 		if (!add_job_to_pattern (rd, job - rd->jobs, start, count)) {
-			log ("Out of pattern space "
-			     "for service 0x%08x (%s)",
-			     par->id, par->label);
+			error (&rd->log,
+			       "Out of decoder pattern space for "
+			       "service 0x%08x (%s).",
+			       par->id, par->label);
 			continue;
 		}
 
@@ -1362,6 +1032,14 @@ vbi3_raw_decoder_add_services	(vbi3_raw_decoder *	rd,
 	return rd->services;
 }
 
+vbi_service_set
+vbi3_raw_decoder_services	(vbi3_raw_decoder *	rd)
+{
+	assert (NULL != rd);
+
+	return rd->services;
+}
+
 /**
  * $param rd Pointer to a vbi3_raw_decoder object allocated with
  *   vbi3_raw_decoder_new().
@@ -1394,7 +1072,7 @@ vbi3_raw_decoder_set_sampling_par
 
 	vbi3_raw_decoder_reset (rd);
 
-	if (!_vbi_sampling_par_verify (sp)) {
+	if (!_vbi_sampling_par_valid_log (sp, &rd->log)) {
 		CLEAR (rd->sampling);
 		return 0;
 	}
@@ -1422,6 +1100,22 @@ vbi3_raw_decoder_get_sampling_par
 	*sp = rd->sampling;
 }
 
+void
+vbi3_raw_decoder_set_log_fn	(vbi3_raw_decoder *	rd,
+				 vbi_log_fn *		log_fn,
+				 void *			user_data,
+				 vbi_log_mask		mask)
+{
+	assert (NULL != rd);
+
+	if (NULL == log_fn)
+		mask = 0;
+
+	rd->log.fn = log_fn;
+	rd->log.user_data = user_data;
+	rd->log.mask = mask;
+}
+
 /**
  * $internal
  *
@@ -1449,8 +1143,15 @@ _vbi3_raw_decoder_init		(vbi3_raw_decoder *	rd,
 
 	vbi3_raw_decoder_reset (rd);
 
+	if (RAW_DECODER_LOG) {
+		vbi3_raw_decoder_set_log_fn (rd,
+					     vbi_log_on_stderr,
+					     /* user_data */ NULL,
+					     /* mask */ VBI_LOG_INFO * 2 - 1);
+	}
+
 	if (NULL != sp) {
-		if (!_vbi_sampling_par_verify (sp))
+		if (!_vbi_sampling_par_valid_log (sp, &rd->log))
 			return FALSE;
 
 		rd->sampling = *sp;
@@ -1496,10 +1197,9 @@ vbi3_raw_decoder_new		(const vbi_sampling_par *sp)
 {
 	vbi3_raw_decoder *rd;
 
-	if (!(rd = malloc (sizeof (*rd)))) {
-		log ("Out of memory");
+	rd = malloc (sizeof (*rd));
+	if (NULL == rd)
 		return NULL;
-	}
 
 	if (!_vbi3_raw_decoder_init (rd, sp)) {
 		free (rd);
