@@ -22,7 +22,7 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: vbi.c,v 1.16 2005/02/17 10:21:17 mschimek Exp $ */
+/* $Id: vbi.c,v 1.20 2006/05/25 08:09:35 mschimek Exp $ */
 
 #include "site_def.h"
 
@@ -64,7 +64,7 @@
  *
  * The ZVBI library provides routines to access raw VBI sampling devices
  * (currently the Linux <a href="http://linux.bytesex.org/v4l2/">V4L and
- * and V4L2</a> API and the FreeBSD, OpenBSD, NetBSD and BSDi
+ * and V4L2</a> API and the FreeBSD
  * <a href="http://telepresence.dmem.strath.ac.uk/bt848/">bktr driver</a> API
  * are supported), a versatile raw VBI bit slicer,
  * decoders for various data services and basic search,
@@ -98,6 +98,36 @@ vbi_init			(void)
 #endif
 }
 
+/**
+ * @ingroup Basic
+ * @param mask Which kind of information to log. Can be @c 0.
+ * @param log_fn This function is called with log messages. Consider
+ *   vbi_log_on_stderr(). Can be @c NULL to disable logging.
+ * @param user_data User pointer passed through to the @a log_fn function.
+ *
+ * Various functions can print warnings, errors and information useful to
+ * debug the library. With this function you can enable these messages
+ * and determine a function to print them.
+ *
+ * @note
+ * The kind and contents of messages logged by particular functions
+ * may change in the future.
+ *
+ * @since 0.2.22
+ */
+void
+vbi_set_log_fn			(vbi_log_mask		mask,
+				 vbi_log_fn *		log_fn,
+				 void *			user_data)
+{
+	if (NULL == log_fn)
+		mask = 0;
+
+	_vbi_global_log.mask		= mask;
+	_vbi_global_log.fn		= log_fn;
+	_vbi_global_log.user_data	= user_data;
+}
+
 /*
  *  Events
  */
@@ -114,7 +144,7 @@ vbi_event_enable(vbi_decoder *vbi, int mask)
 		vbi_teletext_channel_switched(vbi);
 	if (activate & VBI_EVENT_CAPTION)
 		vbi_caption_channel_switched(vbi);
-	if (activate & VBI_EVENT_NETWORK)
+	if (activate & (VBI_EVENT_NETWORK | VBI_EVENT_NETWORK_ID))
 		memset(&vbi->network, 0, sizeof(vbi->network));
 	if (activate & VBI_EVENT_TRIGGER)
 		vbi_trigger_flush(vbi);
@@ -404,11 +434,13 @@ vbi_decode(vbi_decoder *vbi, vbi_sliced *sliced, int lines, double time)
 		  fprintf(stderr, "vbi frame/s dropped at %f, D=%f\n",
 			  time, time - vbi->time);
 
-	  if (vbi->event_mask &
-	      (VBI_EVENT_TTX_PAGE | VBI_EVENT_NETWORK))
+	  if (vbi->event_mask & (VBI_EVENT_TTX_PAGE |
+				 VBI_EVENT_NETWORK |
+				 VBI_EVENT_NETWORK_ID))
 		  vbi_teletext_desync(vbi);
-	  if (vbi->event_mask &
-	      (VBI_EVENT_CAPTION | VBI_EVENT_NETWORK))
+	  if (vbi->event_mask & (VBI_EVENT_CAPTION |
+				 VBI_EVENT_NETWORK |
+				 VBI_EVENT_NETWORK_ID))
 		  vbi_caption_desync(vbi);
 	} else {
 		pthread_mutex_lock(&vbi->chswcd_mutex);
@@ -532,6 +564,8 @@ vbi_channel_switched(vbi_decoder *vbi, vbi_nuid nuid)
 {
 	/* XXX nuid */
 
+	nuid = nuid;
+
 	pthread_mutex_lock(&vbi->chswcd_mutex);
 
 	vbi->chswcd = 1;
