@@ -1,7 +1,7 @@
 /*
  *  libzvbi test
  *
- *  Copyright (C) 2000, 2001, 2002 Michael H. Schimek
+ *  Copyright (C) 2000-2006 Michael H. Schimek
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: capture.c,v 1.19 2005/06/11 22:13:11 mschimek Exp $ */
+/* $Id: capture.c,v 1.24 2006/05/25 08:11:24 mschimek Exp $ */
 
 #undef NDEBUG
 
@@ -37,24 +37,11 @@
 #include <getopt.h>
 #endif
 
-#include "src/libzvbi.h"
-
-vbi_inline int
-vbi_printable			(int			c)
-{
-	if (c < 0)
-		return '?';
-
-	c &= 0x7F;
-
-	if (c < 0x20 || c >= 0x7F)
-		return '.';
-
-	return c;
-}
-
-#define TEST 1
 #include "src/dvb_mux.h"
+#include "src/decoder.h"
+#include "src/io.h"
+#include "src/io-sim.h"
+#include "src/hamm.h"
 
 vbi_capture *		cap;
 vbi_raw_decoder *	par;
@@ -75,8 +62,8 @@ int			bin_ts;
 int			do_read = TRUE;
 int			do_sim;
 int			ignore_error;
-
-#include "sim.c"
+int			desync;
+int			strict;
 
 extern void
 vbi_capture_set_log_fp		(vbi_capture *		capture,
@@ -139,7 +126,7 @@ decode_vps(uint8_t *buf)
 
 	c &= 0x7F;
 
-	label[l] = vbi_printable (c);
+	label[l] = _vbi_to_ascii (c);
 
 	l = (l + 1) % 16;
 
@@ -162,81 +149,6 @@ decode_vps(uint8_t *buf)
 	dump_pil(pil);
 }
 
-static void
-decode_ttx(uint8_t *buf, int line)
-{
-	int packet_address;
-	int magazine, packet;
-	int j;
-
-	packet_address = vbi_unham16p (buf + 0);
-
-	if (packet_address < 0)
-		return; /* hamming error */
-
-	magazine = packet_address & 7;
-	packet = packet_address >> 3;
-
-	if (dump_ttx) {
-		printf("WST %x %02d %03d >", magazine, packet, line);
-
-		for (j = 0; j < 42; j++) {
-			char c = vbi_printable (buf[j]);
-
-			putchar(c);
-		}
-
-		putchar('<');
-		putchar('\n');
-	}
-}
-
-static void
-decode_xds(uint8_t *buf)
-{
-	if (dump_xds) {
-		char c;
-
-		c = vbi_unpar8 (buf[0]);
-		c = vbi_printable (c);
-		putchar(c);
-		c = vbi_unpar8 (buf[1]);
-		c = vbi_printable (c);
-		putchar(c);
-		fflush(stdout);
-	}
-}
-
-static void
-decode_caption(uint8_t *buf, int line)
-{
-	static vbi_bool xds_transport = FALSE;
-	char c = buf[0] & 0x7F;
-
-	if (line >= 280) { /* field 2 */
-		/* 0x01xx..0x0Exx ASCII_or_NUL[0..32] 0x0Fchks */
-		if (vbi_unpar8 (buf[0]) >= 0
-		    && (c >= 0x01 && c <= 0x0F)) {
-			decode_xds(buf);
-			xds_transport = (c != 0x0F);
-		} else if (xds_transport) {
-			decode_xds(buf);
-		}
-
-		return;
-	}
-
-	if (dump_cc) {
-		c = vbi_unpar8 (buf[0]);
-		c = vbi_printable (c);
-		putchar(c);
-		c = vbi_unpar8 (buf[1]);
-		c = vbi_printable (c);
-		putchar(c);
-		fflush(stdout);
-	}
-}
-
 static void
 decode_wss_625(uint8_t *buf)
 {
@@ -324,7 +236,7 @@ decode_sliced(vbi_sliced *s, double time, int lines)
 			putchar(' ');
 
 			for (j = 0; j < sizeof (q->data); ++j) {
-				char c = vbi_printable (q->data[j]);
+				char c = _vbi_to_ascii (q->data[j]);
 				putchar(c);
 			}
 
@@ -338,11 +250,11 @@ decode_sliced(vbi_sliced *s, double time, int lines)
 		} else if (s->id & VBI_SLICED_VPS) {
 			decode_vps(s->data);
 		} else if (s->id & VBI_SLICED_TELETEXT_B) {
-			decode_ttx(s->data, s->line);
+			/* Use ./decode instead. */
 		} else if (s->id & VBI_SLICED_CAPTION_525) {
-			decode_caption(s->data, s->line);
+			/* Use ./decode instead. */
 		} else if (s->id & VBI_SLICED_CAPTION_625) {
-			decode_caption(s->data, s->line);
+			/* Use ./decode instead. */
 		} else if (s->id & VBI_SLICED_WSS_625) {
 			decode_wss_625(s->data);
 		} else if (s->id & VBI_SLICED_WSS_CPR1204) {
@@ -431,10 +343,7 @@ mainloop(void)
 	for (quit = FALSE; !quit;) {
 		int r;
 
-		if (do_sim) {
-			read_sim(raw, sliced, &lines, &timestamp);
-			r = 1;
-		} else if (do_read) {
+		if (do_read) {
 			r = vbi_capture_read(cap, raw, sliced,
 					     &lines, &timestamp, &tv);
 		} else {
@@ -488,11 +397,12 @@ mainloop(void)
 	}
 }
 
-static const char short_options[] = "123d:elnpstvPT";
+static const char short_options[] = "123cd:elnpr:stvPT";
 
 #ifdef HAVE_GETOPT_LONG
 static const struct option
 long_options[] = {
+	{ "desync",	no_argument,		NULL,		'c' },
 	{ "device",	required_argument,	NULL,		'd' },
 	{ "ignore-error", no_argument,		NULL,		'e' },
 	{ "pid",	required_argument,	NULL,		'i' },
@@ -507,6 +417,7 @@ long_options[] = {
 	{ "ts",		no_argument,		NULL,		'T' },
 	{ "read",	no_argument,		&do_read,	TRUE },
 	{ "pull",	no_argument,		&do_read,	FALSE },
+	{ "strict",	required_argument,	NULL,		'r' },
 	{ "sim",	no_argument,		NULL,		's' },
 	{ "ntsc",	no_argument,		NULL,		'n' },
 	{ "pal",	no_argument,		NULL,		'p' },
@@ -549,6 +460,9 @@ main(int argc, char **argv)
 		case '3':
 			interface = c - '0';
 			break;
+		case 'c':
+			desync ^= TRUE;
+			break;
 		case 'd':
 			free (dev_name);
 			dev_name = strdup (optarg);
@@ -572,6 +486,13 @@ main(int argc, char **argv)
 		case 'P':
 			bin_pes ^= TRUE;
 			break;
+		case 'r':
+			strict = strtol (optarg, NULL, 0);
+			if (strict < -1)
+				strict = -1;
+			else if (strict > 2)
+				strict = 2;
+			break;
 		case 's':
 			do_sim ^= TRUE;
 			break;
@@ -589,18 +510,24 @@ main(int argc, char **argv)
 			exit(EXIT_FAILURE);
 		}
 
-	dump = dump_ttx | dump_xds | dump_cc
-		| dump_wss | dump_vps | dump_sliced;
+	if (dump_ttx | dump_cc | dump_xds) {
+		fprintf (stderr, "\
+Teletext, CC and XDS decoding are no longer supported by this tool.\n\
+Run  ./capture --sliced | ./decode --ttx --cc --xds  instead.\n");
+		exit (EXIT_FAILURE);
+	}
+
+	dump = dump_wss | dump_vps | dump_sliced;
 
 	services = VBI_SLICED_VBI_525 | VBI_SLICED_VBI_625
 		| VBI_SLICED_TELETEXT_B | VBI_SLICED_CAPTION_525
 		| VBI_SLICED_CAPTION_625 | VBI_SLICED_VPS
 		| VBI_SLICED_WSS_625 | VBI_SLICED_WSS_CPR1204;
 
-	strict = 0;
-
 	if (do_sim) {
-		par = init_sim (scanning, services);
+		cap = vbi_capture_sim_new (scanning, &services,
+					   /* interlaced */ FALSE, !desync);
+		assert ((par = vbi_capture_parameters(cap)));
 	} else {
 		do {
 			if (-1 != pid) {
