@@ -19,7 +19,7 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: osc.c,v 1.20 2005/05/11 20:16:02 mschimek Exp $ */
+/* $Id: osc.c,v 1.27 2006/05/28 20:15:59 mschimek Exp $ */
 
 #undef NDEBUG
 
@@ -37,9 +37,11 @@
 #include <getopt.h>
 #endif
 
-#include "src/libzvbi.h"
-
-#define TEST 1
+#include "src/io.h"
+#include "src/decoder.h"
+#include "src/misc.h"
+#include "src/hamm.h"
+#include "src/io-sim.h"
 #include "src/raw_decoder.h"	/* _vbi_service_table[] */
 
 #ifndef X_DISPLAY_MISSING
@@ -57,6 +59,7 @@ vbi_bool		quit;
 
 int			do_sim;
 int			ignore_error;
+int			desync;
 
 Display *		display;
 int			screen;
@@ -74,22 +77,6 @@ int			draw_row, draw_offset;
 int			draw_count = -1;
 int                     cur_x, cur_y;
 
-#include "sim.c"
-
-vbi_inline int
-vbi_printable			(int			c)
-{
-	if (c < 0)
-		return '?';
-
-	c &= 0x7F;
-
-	if (c < 0x20 || c >= 0x7F)
-		return '.';
-
-	return c;
-}
-
 extern void
 vbi_capture_set_log_fp		(vbi_capture *		capture,
 				 FILE *			fp);
@@ -120,7 +107,7 @@ decode_ttx(uint8_t *buf, int line)
         text += sprintf(text, "pg %x%02d ln %03d >", magazine, packet, line);
 
         for (j = 0; j < 42; j++) {
-	   char c = vbi_printable (buf[j]);
+	   char c = _vbi_to_ascii (buf[j]);
 	   
 	   *text = c;
 	   text++;
@@ -186,7 +173,7 @@ decode_vps(uint8_t *buf)
 
 	c &= 0x7F;
 
-	label[l] = vbi_printable (c);
+	label[l] = _vbi_to_ascii (c);
 
 	l = (l + 1) % 16;
 
@@ -218,7 +205,7 @@ static void
 draw(unsigned char *raw)
 {
 	int rem = src_w - draw_offset;
-	unsigned char buf[256];
+	char buf[256];
 	unsigned char *data = raw;
 	int i, v, h0, field, end, line;
         XTextItem xti;
@@ -276,7 +263,7 @@ draw(unsigned char *raw)
 	}
 
 	for (i = 0; i < slines; i++)
-		if (sliced[i].line == line)
+		if (sliced[i].line == (unsigned int) line)
 			break;
 	if (i < slines) {
 	   int svc_idx=0;
@@ -354,7 +341,9 @@ draw(unsigned char *raw)
 	xti.font = 0;
 
 	XDrawText(display, window, gc, 4, src_h + 12, &xti, 1);
-        xti.nchars = snprintf(buf, 255, "(%d, %3.0d)", cur_x+draw_offset, (1000*(dst_h-cur_y))/(dst_h-src_h));
+        xti.nchars = snprintf(buf, 255, "(%d, %3d)", cur_x+draw_offset,
+	  (dst_h - cur_y) * 256 / v);
+//	  (1000*(dst_h-cur_y))/(dst_h-src_h));
         XDrawText(display, window, gc, 4, src_h + 24, &xti, 1);
 
 	data = raw + draw_offset + draw_row * src_w;
@@ -442,13 +431,16 @@ redraw:
 }
 
 static void
-init_window(int ac, char **av, char *dev_name)
+init_window(int ac, char **av, const char *dev_name)
 {
 	char buf[256];
 	Atom delete_window_atom;
 	XWindowAttributes wa;
 	int i;
 
+	ac = ac;
+	av = av;
+
 	if (!(display = XOpenDisplay(NULL))) {
 		fprintf(stderr, "No display\n");
 		exit(EXIT_FAILURE);
@@ -561,13 +553,8 @@ mainloop(void)
 	for (quit = FALSE; !quit;) {
 		int r;
 
-		if (do_sim) {
-			read_sim(raw1, sliced, &slines, &timestamp);
-			r = 1;
-		} else {
-			r = vbi_capture_read(cap, raw1, sliced,
-					     &slines, &timestamp, &tv);
-		}
+		r = vbi_capture_read(cap, raw1, sliced,
+				     &slines, &timestamp, &tv);
 
 		switch (r) {
 		case -1:
@@ -599,11 +586,12 @@ mainloop(void)
 	}
 }
 
-static const char short_options[] = "123d:enpsv";
+static const char short_options[] = "123cd:enpsv";
 
 #ifdef HAVE_GETOPT_LONG
 static const struct option
 long_options[] = {
+	{ "desync",	no_argument,		NULL,		'c' },
 	{ "device",	required_argument,	NULL,		'd' },
 	{ "ignore-error", no_argument,		NULL,		'e' },
 	{ "ntsc",	no_argument,		NULL,		'n' },
@@ -622,7 +610,7 @@ long_options[] = {
 int
 main(int argc, char **argv)
 {
-	char *dev_name = "/dev/vbi";
+	const char *dev_name = "/dev/vbi";
 	char *errstr;
 	unsigned int services;
 	int scanning = 625;
@@ -644,6 +632,9 @@ main(int argc, char **argv)
 		case '3':
 			interface = c - '0';
 			break;
+		case 'c':
+			desync ^= TRUE;
+			break;
 		case 'd':
 			dev_name = strdup (optarg);
 			break;
@@ -675,7 +666,9 @@ main(int argc, char **argv)
 	strict = 0;
 
 	if (do_sim) {
-		par = init_sim (scanning, services);
+		cap = vbi_capture_sim_new (scanning, &services,
+					   /* interlaced */ FALSE, !desync);
+		assert ((par = vbi_capture_parameters(cap)));
 	} else {
 		do {
 			if (1 != interface) {
